// Copyright 2002-2011, University of Colorado
/**
 * Base class for States of Matter simulations.  This was created primarily to
 * avoid duplication of code between the first States of Matter flavor and
 * the newer "basics" flavor.
 *
 * @author John Blanco
 */
define( function( require ) {
'use strict';
  
  // modules
var inherit = require( 'PHET_CORE/inherit' );
var ActionEvent = require('java.awt.event.ActionEvent');
var ActionListener = require('java.awt.event.ActionListener');
var JMenu = require('javax.swing.JMenu');
var JRadioButtonMenuItem = require('javax.swing.JRadioButtonMenuItem');
var PhetApplicationConfig = require('edu.colorado.phet.common.phetcommon.application.PhetApplicationConfig');
var VoidFunction1 = require('edu.colorado.phet.common.phetcommon.util.function.VoidFunction1');
var TeacherMenu = require('edu.colorado.phet.common.phetcommon.view.menu.TeacherMenu');
var PiccoloPhetApplication = require('edu.colorado.phet.common.piccolophet.PiccoloPhetApplication');
var StatesOfMatterGlobalState = require('STATES_OF_MATTER/states-of-matter/StatesOfMatterGlobalState');
var StatesOfMatterStrings = require('STATES_OF_MATTER/states-of-matter/StatesOfMatterStrings');
var DeveloperControlsMenuItem = require('STATES_OF_MATTER/states-of-matter/developer/DeveloperControlsMenuItem');
 
function AbstractStatesOfMatterApplication( config) {
    PiccoloPhetApplication.call(this, config);
    initMenubar();
}
  
  return inherit(PiccoloPhetApplication,AbstractStatesOfMatterApplication,{
//----------------------------------------------------------------------------
// Initialization
//----------------------------------------------------------------------------
/**
     * Initializes the menu bar.
     */
initMenubar: function() {
    // Options menu
    var kelvinRadioButton = new JRadioButtonMenuItem(StatesOfMatterStrings.KELVIN).withAnonymousClassBody( {
        initializer:function(){
            addActionListener(new ActionListener().withAnonymousClassBody( {
                actionPerformed: function( e) {
                    StatesOfMatterGlobalState.temperatureUnitsProperty.set(TemperatureUnits.KELVIN);
                }
            }));
        }
    });
    var celsiusRadioButton = new JRadioButtonMenuItem(StatesOfMatterStrings.CELSIUS).withAnonymousClassBody( {
        initializer:function(){
            addActionListener(new ActionListener().withAnonymousClassBody( {
                actionPerformed: function( e) {
                    StatesOfMatterGlobalState.temperatureUnitsProperty.set(TemperatureUnits.CELSIUS);
                }
            }));
        }
    });
    StatesOfMatterGlobalState.temperatureUnitsProperty.addObserver(new VoidFunction1().withAnonymousClassBody( {
        apply: function( temperatureUnitsValue) {
            kelvinRadioButton.setSelected(temperatureUnitsValue == TemperatureUnits.KELVIN);
            celsiusRadioButton.setSelected(temperatureUnitsValue == TemperatureUnits.CELSIUS);
        }
    }));
    // for use in making handouts, on projectors, etc.
    getPhetFrame().addMenu(new TeacherMenu().withAnonymousClassBody( {
        initializer:function(){
            add(kelvinRadioButton);
            add(celsiusRadioButton);
            addSeparator();
            addWhiteBackgroundMenuItem(StatesOfMatterGlobalState.whiteBackground);
        }
    }));
    // Developer menu
    var developerMenu = getPhetFrame().getDeveloperMenu();
    developerMenu.add(new DeveloperControlsMenuItem(this));
}
});
} );

