// Copyright 2002-2011, University of Colorado
/**
 * ModelViewTransform provides the transforms between model and view coordinate systems.
 *
 * @author John Blanco
 * @deprecated please use edu.colorado.phet.common.phetcommon.view.graphics.transforms.ModelViewTransform
 */
define( function( require ) {
'use strict';
  
  // modules
var inherit = require( 'PHET_CORE/inherit' );
var AffineTransform = require('java.awt.geom.AffineTransform');
var Vector2 = require('java.awt.geom.Vector2');
var Rectangle = require('KITE/Rectangle');
 
/**
     * Constructor.
     *
     * @param xScale    x scale for mapping from model to view
     * @param yScale    y scale for mapping from model to view
     * @param xOffset   x translation for mapping from model to view
     * @param yOffset   y translation for mapping from model to view
     * @param flipSignX if true, flip the sign on the x component
     * @param flipSignY if true, flip the sign on the y component
     */
function ModelViewTransform( xScale,  yScale,  xOffset,  yOffset,  flipSignX,  flipSignY) {
    //----------------------------------------------------------------------------
    // Instance Data
    //----------------------------------------------------------------------------
     
  //private
this._modelToViewTransform;
     
  //private
this._viewToModelTransform;
     
  //private
this._flipSignX, _flipSignY;
    _flipSignX = flipSignX;
    _flipSignY = flipSignY;
    _modelToViewTransform = new AffineTransform();
    _modelToViewTransform.scale(xScale, yScale);
    _modelToViewTransform.translate(xOffset, yOffset);
    _viewToModelTransform = new AffineTransform();
    _viewToModelTransform.translate(-xOffset, -yOffset);
    _viewToModelTransform.scale(1d / xScale, 1d / yScale);
}
function ModelViewTransform( xScale,  yScale,  xOffset,  yOffset) {
    //----------------------------------------------------------------------------
    // Instance Data
    //----------------------------------------------------------------------------
     
  //private
this._modelToViewTransform;
     
  //private
this._viewToModelTransform;
     
  //private
this._flipSignX, _flipSignY;
    this(xScale, yScale, xOffset, yOffset, false, false);
}
/**
     * Constructs an identity transform.
     */
function ModelViewTransform() {
    //----------------------------------------------------------------------------
    // Instance Data
    //----------------------------------------------------------------------------
     
  //private
this._modelToViewTransform;
     
  //private
this._viewToModelTransform;
     
  //private
this._flipSignX, _flipSignY;
    this(1, 1, 0, 0, false, false);
}
  
  return inherit(Object,ModelViewTransform,{
//----------------------------------------------------------------------------
// Model-to-view transforms
//----------------------------------------------------------------------------
/**
     * Maps a point from model to view coordinates.
     * If pView is not null, the result is returned in pView.
     *
     * @param pModel point in model coordinates
     * @param pView  point in view coordinates, possibly null
     * @return point in view coordinates
     */
modelToView: function( pModel,  pView) {
    var p = _modelToViewTransform.transform(pModel, pView);
    if (_flipSignX) {
        p.setLocation(-p.getX(), p.getY());
    }
    if (_flipSignY) {
        p.setLocation(p.getX(), -p.getY());
    }
    return p;
},
modelToView: function( pModel) {
    return modelToView(pModel, null);
},
modelToView: function( xModel,  yModel,  pView) {
    return modelToView(new Vector2(xModel, yModel), pView);
},
modelToView: function( xModel,  yModel) {
    return modelToView(xModel, yModel, null);
},
/**
     * Maps a rectangle from model to view coordinates.
     *
     * @param rModel
     * @return Rectangle2D in view coordinates
     */
modelToView: function( rModel) {
    var r = _modelToViewTransform.createTransformedShape(rModel).getBounds2D();
    if (_flipSignX) {
        r.setRect(-r.getWidth() - r.getX(), r.getY(), r.getWidth(), r.getHeight());
    }
    if (_flipSignY) {
        r.setRect(r.getX(), -r.getHeight() - r.getY(), r.getWidth(), r.getHeight());
    }
    return r;
},
//----------------------------------------------------------------------------
// View-to-model transforms
//----------------------------------------------------------------------------
/**
     * Maps a point from view to model coordinates.
     * If pModel is not null, the result is returned in pModel.
     *
     * @param pView  point in view coordinates
     * @param pModel point in model coordinates, possibly null
     * @return point in model coordinates
     */
viewToModel: function( pView,  pModel) {
    var p = _viewToModelTransform.transform(pView, pModel);
    if (_flipSignX) {
        p.setLocation(-p.getX(), p.getY());
    }
    if (_flipSignY) {
        p.setLocation(p.getX(), -p.getY());
    }
    return p;
},
viewToModel: function( pView) {
    return viewToModel(pView, null);
},
viewToModel: function( xView,  yView,  pModel) {
    return viewToModel(new Vector2(xView, yView), pModel);
},
viewToModel: function( xView,  yView) {
    return viewToModel(new Vector2(xView, yView));
},
/**
     * Maps a rectangle from view to model to coordinates.
     *
     * @param rView
     * @param Rectangle2D in model coordinates
     */
viewToModel: function( rView) {
    var r = _viewToModelTransform.createTransformedShape(rView).getBounds2D();
    if (_flipSignX) {
        r.setRect(-r.getX(), r.getY(), r.getWidth(), r.getHeight());
    }
    if (_flipSignY) {
        r.setRect(r.getX(), -r.getY(), r.getWidth(), r.getHeight());
    }
    return r;
}
});
} );

