// Copyright 2002-2011, University of Colorado
/**
 * This class is the slider that is used to control the StoveNode, causing it
 * to add heat or cooling to the simulated system.
 *
 * @author Sam Reid
 * @author John Blanco
 */
define( function( require ) {
'use strict';
  
  // modules
var inherit = require( 'PHET_CORE/inherit' );
var BasicStroke = require('java.awt.BasicStroke');
var Color = require( 'SCENERY/util/Color' );
var Font = require('SCENERY/util/Font');
var GradientPaint = require('java.awt.GradientPaint');
var Stroke = require('java.awt.Stroke');
var Property = require( 'AXON/Property' );
var SettableProperty = require('edu.colorado.phet.common.phetcommon.model.property.SettableProperty');
var DoubleGeneralPath = require('edu.colorado.phet.common.phetcommon.view.util.DoubleGeneralPath');
var PhetFont = require('edu.colorado.phet.common.phetcommon.view.util.PhetFont');
var PhetPPath = require('edu.colorado.phet.common.piccolophet.nodes.PhetPPath');
var PhetPText = require('edu.colorado.phet.common.piccolophet.nodes.PhetPText');
var VSliderNode = require('edu.colorado.phet.common.piccolophet.nodes.slider.VSliderNode');
var UserComponents = require('STATES_OF_MATTER/states-of-matter/StatesOfMatterSimSharing/UserComponents');
var StatesOfMatterStrings = require('STATES_OF_MATTER/states-of-matter/StatesOfMatterStrings');
var Node = require('SCENERY/nodes/Node');
var PBasicInputEventHandler = require('edu.umd.cs.piccolo.event.PBasicInputEventHandler');
var PInputEvent = require('edu.umd.cs.piccolo.event.PInputEvent');
 
// Meant to look warm.
 
  //private
var TOP_SIDE_TRACK_COLOR = new Color(255, 69, 0);
// Meant to look cold.
 
  //private
var BOTTOM_SIDE_TRACK_COLOR = new Color(0, 0, 240);
 
  //private
var LABEL_FONT = new PhetFont(20, true);
  
  // static class: TickMark
var TickMark = 
// Convenience class for creating a tick mark that works for this slider.
 
  //private
define( function( require ) {
 
  //private
var INDENT = 4;
 
  //private
var LENGTH = 10;
 
  //private
var STROKE_WIDTH = 2;
 
  //private
var STROKE = new BasicStroke(STROKE_WIDTH);
 
  //private
function TickMark() {
    var path = new DoubleGeneralPath(INDENT, STROKE_WIDTH / 2).withAnonymousClassBody( {
        initializer:function(){
            lineTo(INDENT + LENGTH, STROKE_WIDTH / 2);
        }
    });
    addChild(new PhetPPath(path.getGeneralPath(), STROKE, Color.BLACK));
}
  
  return inherit(Node,TickMark,{
});
} );;
function StoveControlSliderNode( value) {
    VSliderNode.call(this, UserComponents.stoveSlider, -1, 1, 6, 75, value, new BooleanProperty(true));
    // Show labels for add, zero, and remove.
    addLabel(+1, new PhetPText(StatesOfMatterStrings.STOVE_CONTROL_PANEL_HEAT_LABEL, LABEL_FONT));
    addLabel(0.0, new TickMark());
    addLabel(-1, new PhetPText(StatesOfMatterStrings.STOVE_CONTROL_PANEL_COOL_LABEL, LABEL_FONT));
    // Return to 0 when the user releases the slider.
    addInputEventListener(new PBasicInputEventHandler().withAnonymousClassBody( {
        mouseReleased: function( event) {
            value.set(0.0);
        }
    }));
    // indicate the heat/coolness setting.
    setTrackFillPaint(new GradientPaint(0, 0, TOP_SIDE_TRACK_COLOR, 0, trackLength, BOTTOM_SIDE_TRACK_COLOR, false));
}
  
  return inherit(VSliderNode,StoveControlSliderNode,{
});
} );

